"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
require("mocha");
const extra_utils_1 = require("../../../../shared/extra-utils");
const check_api_params_1 = require("../../../../shared/extra-utils/requests/check-api-params");
const http_error_codes_1 = require("../../../../shared/core-utils/miscs/http-error-codes");
describe('Test blocklist API validators', function () {
    let servers;
    let server;
    let userAccessToken;
    before(function () {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            this.timeout(60000);
            servers = yield extra_utils_1.flushAndRunMultipleServers(2);
            yield extra_utils_1.setAccessTokensToServers(servers);
            server = servers[0];
            const user = { username: 'user1', password: 'password' };
            yield extra_utils_1.createUser({ url: server.url, accessToken: server.accessToken, username: user.username, password: user.password });
            userAccessToken = yield extra_utils_1.userLogin(server, user);
            yield extra_utils_1.doubleFollow(servers[0], servers[1]);
        });
    });
    describe('When managing user blocklist', function () {
        describe('When managing user accounts blocklist', function () {
            const path = '/api/v1/users/me/blocklist/accounts';
            describe('When listing blocked accounts', function () {
                it('Should fail with an unauthenticated user', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield extra_utils_1.makeGetRequest({
                            url: server.url,
                            path,
                            statusCodeExpected: http_error_codes_1.HttpStatusCode.UNAUTHORIZED_401
                        });
                    });
                });
                it('Should fail with a bad start pagination', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield check_api_params_1.checkBadStartPagination(server.url, path, server.accessToken);
                    });
                });
                it('Should fail with a bad count pagination', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield check_api_params_1.checkBadCountPagination(server.url, path, server.accessToken);
                    });
                });
                it('Should fail with an incorrect sort', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield check_api_params_1.checkBadSortPagination(server.url, path, server.accessToken);
                    });
                });
            });
            describe('When blocking an account', function () {
                it('Should fail with an unauthenticated user', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield extra_utils_1.makePostBodyRequest({
                            url: server.url,
                            path,
                            fields: { accountName: 'user1' },
                            statusCodeExpected: http_error_codes_1.HttpStatusCode.UNAUTHORIZED_401
                        });
                    });
                });
                it('Should fail with an unknown account', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield extra_utils_1.makePostBodyRequest({
                            url: server.url,
                            token: server.accessToken,
                            path,
                            fields: { accountName: 'user2' },
                            statusCodeExpected: http_error_codes_1.HttpStatusCode.NOT_FOUND_404
                        });
                    });
                });
                it('Should fail to block ourselves', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield extra_utils_1.makePostBodyRequest({
                            url: server.url,
                            token: server.accessToken,
                            path,
                            fields: { accountName: 'root' },
                            statusCodeExpected: http_error_codes_1.HttpStatusCode.CONFLICT_409
                        });
                    });
                });
                it('Should succeed with the correct params', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield extra_utils_1.makePostBodyRequest({
                            url: server.url,
                            token: server.accessToken,
                            path,
                            fields: { accountName: 'user1' },
                            statusCodeExpected: http_error_codes_1.HttpStatusCode.NO_CONTENT_204
                        });
                    });
                });
            });
            describe('When unblocking an account', function () {
                it('Should fail with an unauthenticated user', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield extra_utils_1.makeDeleteRequest({
                            url: server.url,
                            path: path + '/user1',
                            statusCodeExpected: http_error_codes_1.HttpStatusCode.UNAUTHORIZED_401
                        });
                    });
                });
                it('Should fail with an unknown account block', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield extra_utils_1.makeDeleteRequest({
                            url: server.url,
                            path: path + '/user2',
                            token: server.accessToken,
                            statusCodeExpected: http_error_codes_1.HttpStatusCode.NOT_FOUND_404
                        });
                    });
                });
                it('Should succeed with the correct params', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield extra_utils_1.makeDeleteRequest({
                            url: server.url,
                            path: path + '/user1',
                            token: server.accessToken,
                            statusCodeExpected: http_error_codes_1.HttpStatusCode.NO_CONTENT_204
                        });
                    });
                });
            });
        });
        describe('When managing user servers blocklist', function () {
            const path = '/api/v1/users/me/blocklist/servers';
            describe('When listing blocked servers', function () {
                it('Should fail with an unauthenticated user', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield extra_utils_1.makeGetRequest({
                            url: server.url,
                            path,
                            statusCodeExpected: http_error_codes_1.HttpStatusCode.UNAUTHORIZED_401
                        });
                    });
                });
                it('Should fail with a bad start pagination', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield check_api_params_1.checkBadStartPagination(server.url, path, server.accessToken);
                    });
                });
                it('Should fail with a bad count pagination', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield check_api_params_1.checkBadCountPagination(server.url, path, server.accessToken);
                    });
                });
                it('Should fail with an incorrect sort', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield check_api_params_1.checkBadSortPagination(server.url, path, server.accessToken);
                    });
                });
            });
            describe('When blocking a server', function () {
                it('Should fail with an unauthenticated user', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield extra_utils_1.makePostBodyRequest({
                            url: server.url,
                            path,
                            fields: { host: 'localhost:9002' },
                            statusCodeExpected: http_error_codes_1.HttpStatusCode.UNAUTHORIZED_401
                        });
                    });
                });
                it('Should succeed with an unknown server', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield extra_utils_1.makePostBodyRequest({
                            url: server.url,
                            token: server.accessToken,
                            path,
                            fields: { host: 'localhost:9003' },
                            statusCodeExpected: http_error_codes_1.HttpStatusCode.NO_CONTENT_204
                        });
                    });
                });
                it('Should fail with our own server', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield extra_utils_1.makePostBodyRequest({
                            url: server.url,
                            token: server.accessToken,
                            path,
                            fields: { host: 'localhost:' + server.port },
                            statusCodeExpected: http_error_codes_1.HttpStatusCode.CONFLICT_409
                        });
                    });
                });
                it('Should succeed with the correct params', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield extra_utils_1.makePostBodyRequest({
                            url: server.url,
                            token: server.accessToken,
                            path,
                            fields: { host: 'localhost:' + servers[1].port },
                            statusCodeExpected: http_error_codes_1.HttpStatusCode.NO_CONTENT_204
                        });
                    });
                });
            });
            describe('When unblocking a server', function () {
                it('Should fail with an unauthenticated user', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield extra_utils_1.makeDeleteRequest({
                            url: server.url,
                            path: path + '/localhost:' + servers[1].port,
                            statusCodeExpected: http_error_codes_1.HttpStatusCode.UNAUTHORIZED_401
                        });
                    });
                });
                it('Should fail with an unknown server block', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield extra_utils_1.makeDeleteRequest({
                            url: server.url,
                            path: path + '/localhost:9004',
                            token: server.accessToken,
                            statusCodeExpected: http_error_codes_1.HttpStatusCode.NOT_FOUND_404
                        });
                    });
                });
                it('Should succeed with the correct params', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield extra_utils_1.makeDeleteRequest({
                            url: server.url,
                            path: path + '/localhost:' + servers[1].port,
                            token: server.accessToken,
                            statusCodeExpected: http_error_codes_1.HttpStatusCode.NO_CONTENT_204
                        });
                    });
                });
            });
        });
    });
    describe('When managing server blocklist', function () {
        describe('When managing server accounts blocklist', function () {
            const path = '/api/v1/server/blocklist/accounts';
            describe('When listing blocked accounts', function () {
                it('Should fail with an unauthenticated user', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield extra_utils_1.makeGetRequest({
                            url: server.url,
                            path,
                            statusCodeExpected: http_error_codes_1.HttpStatusCode.UNAUTHORIZED_401
                        });
                    });
                });
                it('Should fail with a user without the appropriate rights', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield extra_utils_1.makeGetRequest({
                            url: server.url,
                            token: userAccessToken,
                            path,
                            statusCodeExpected: http_error_codes_1.HttpStatusCode.FORBIDDEN_403
                        });
                    });
                });
                it('Should fail with a bad start pagination', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield check_api_params_1.checkBadStartPagination(server.url, path, server.accessToken);
                    });
                });
                it('Should fail with a bad count pagination', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield check_api_params_1.checkBadCountPagination(server.url, path, server.accessToken);
                    });
                });
                it('Should fail with an incorrect sort', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield check_api_params_1.checkBadSortPagination(server.url, path, server.accessToken);
                    });
                });
            });
            describe('When blocking an account', function () {
                it('Should fail with an unauthenticated user', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield extra_utils_1.makePostBodyRequest({
                            url: server.url,
                            path,
                            fields: { accountName: 'user1' },
                            statusCodeExpected: http_error_codes_1.HttpStatusCode.UNAUTHORIZED_401
                        });
                    });
                });
                it('Should fail with a user without the appropriate rights', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield extra_utils_1.makePostBodyRequest({
                            url: server.url,
                            token: userAccessToken,
                            path,
                            fields: { accountName: 'user1' },
                            statusCodeExpected: http_error_codes_1.HttpStatusCode.FORBIDDEN_403
                        });
                    });
                });
                it('Should fail with an unknown account', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield extra_utils_1.makePostBodyRequest({
                            url: server.url,
                            token: server.accessToken,
                            path,
                            fields: { accountName: 'user2' },
                            statusCodeExpected: http_error_codes_1.HttpStatusCode.NOT_FOUND_404
                        });
                    });
                });
                it('Should fail to block ourselves', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield extra_utils_1.makePostBodyRequest({
                            url: server.url,
                            token: server.accessToken,
                            path,
                            fields: { accountName: 'root' },
                            statusCodeExpected: http_error_codes_1.HttpStatusCode.CONFLICT_409
                        });
                    });
                });
                it('Should succeed with the correct params', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield extra_utils_1.makePostBodyRequest({
                            url: server.url,
                            token: server.accessToken,
                            path,
                            fields: { accountName: 'user1' },
                            statusCodeExpected: http_error_codes_1.HttpStatusCode.NO_CONTENT_204
                        });
                    });
                });
            });
            describe('When unblocking an account', function () {
                it('Should fail with an unauthenticated user', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield extra_utils_1.makeDeleteRequest({
                            url: server.url,
                            path: path + '/user1',
                            statusCodeExpected: http_error_codes_1.HttpStatusCode.UNAUTHORIZED_401
                        });
                    });
                });
                it('Should fail with a user without the appropriate rights', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield extra_utils_1.makeDeleteRequest({
                            url: server.url,
                            path: path + '/user1',
                            token: userAccessToken,
                            statusCodeExpected: http_error_codes_1.HttpStatusCode.FORBIDDEN_403
                        });
                    });
                });
                it('Should fail with an unknown account block', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield extra_utils_1.makeDeleteRequest({
                            url: server.url,
                            path: path + '/user2',
                            token: server.accessToken,
                            statusCodeExpected: http_error_codes_1.HttpStatusCode.NOT_FOUND_404
                        });
                    });
                });
                it('Should succeed with the correct params', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield extra_utils_1.makeDeleteRequest({
                            url: server.url,
                            path: path + '/user1',
                            token: server.accessToken,
                            statusCodeExpected: http_error_codes_1.HttpStatusCode.NO_CONTENT_204
                        });
                    });
                });
            });
        });
        describe('When managing server servers blocklist', function () {
            const path = '/api/v1/server/blocklist/servers';
            describe('When listing blocked servers', function () {
                it('Should fail with an unauthenticated user', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield extra_utils_1.makeGetRequest({
                            url: server.url,
                            path,
                            statusCodeExpected: http_error_codes_1.HttpStatusCode.UNAUTHORIZED_401
                        });
                    });
                });
                it('Should fail with a user without the appropriate rights', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield extra_utils_1.makeGetRequest({
                            url: server.url,
                            token: userAccessToken,
                            path,
                            statusCodeExpected: http_error_codes_1.HttpStatusCode.FORBIDDEN_403
                        });
                    });
                });
                it('Should fail with a bad start pagination', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield check_api_params_1.checkBadStartPagination(server.url, path, server.accessToken);
                    });
                });
                it('Should fail with a bad count pagination', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield check_api_params_1.checkBadCountPagination(server.url, path, server.accessToken);
                    });
                });
                it('Should fail with an incorrect sort', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield check_api_params_1.checkBadSortPagination(server.url, path, server.accessToken);
                    });
                });
            });
            describe('When blocking a server', function () {
                it('Should fail with an unauthenticated user', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield extra_utils_1.makePostBodyRequest({
                            url: server.url,
                            path,
                            fields: { host: 'localhost:' + servers[1].port },
                            statusCodeExpected: http_error_codes_1.HttpStatusCode.UNAUTHORIZED_401
                        });
                    });
                });
                it('Should fail with a user without the appropriate rights', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield extra_utils_1.makePostBodyRequest({
                            url: server.url,
                            token: userAccessToken,
                            path,
                            fields: { host: 'localhost:' + servers[1].port },
                            statusCodeExpected: http_error_codes_1.HttpStatusCode.FORBIDDEN_403
                        });
                    });
                });
                it('Should succeed with an unknown server', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield extra_utils_1.makePostBodyRequest({
                            url: server.url,
                            token: server.accessToken,
                            path,
                            fields: { host: 'localhost:9003' },
                            statusCodeExpected: http_error_codes_1.HttpStatusCode.NO_CONTENT_204
                        });
                    });
                });
                it('Should fail with our own server', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield extra_utils_1.makePostBodyRequest({
                            url: server.url,
                            token: server.accessToken,
                            path,
                            fields: { host: 'localhost:' + server.port },
                            statusCodeExpected: http_error_codes_1.HttpStatusCode.CONFLICT_409
                        });
                    });
                });
                it('Should succeed with the correct params', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield extra_utils_1.makePostBodyRequest({
                            url: server.url,
                            token: server.accessToken,
                            path,
                            fields: { host: 'localhost:' + servers[1].port },
                            statusCodeExpected: http_error_codes_1.HttpStatusCode.NO_CONTENT_204
                        });
                    });
                });
            });
            describe('When unblocking a server', function () {
                it('Should fail with an unauthenticated user', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield extra_utils_1.makeDeleteRequest({
                            url: server.url,
                            path: path + '/localhost:' + servers[1].port,
                            statusCodeExpected: http_error_codes_1.HttpStatusCode.UNAUTHORIZED_401
                        });
                    });
                });
                it('Should fail with a user without the appropriate rights', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield extra_utils_1.makeDeleteRequest({
                            url: server.url,
                            path: path + '/localhost:' + servers[1].port,
                            token: userAccessToken,
                            statusCodeExpected: http_error_codes_1.HttpStatusCode.FORBIDDEN_403
                        });
                    });
                });
                it('Should fail with an unknown server block', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield extra_utils_1.makeDeleteRequest({
                            url: server.url,
                            path: path + '/localhost:9004',
                            token: server.accessToken,
                            statusCodeExpected: http_error_codes_1.HttpStatusCode.NOT_FOUND_404
                        });
                    });
                });
                it('Should succeed with the correct params', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield extra_utils_1.makeDeleteRequest({
                            url: server.url,
                            path: path + '/localhost:' + servers[1].port,
                            token: server.accessToken,
                            statusCodeExpected: http_error_codes_1.HttpStatusCode.NO_CONTENT_204
                        });
                    });
                });
            });
        });
    });
    after(function () {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield extra_utils_1.cleanupTests(servers);
        });
    });
});
